Unit SF33R_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc, UTSF10_p;

procedure SF33R(X :Real; N :Integer; NMAX :Integer;
                var FUN1 :Array of Real; var FUN2 :Array of Real;
                var WORK :Array of Real; var IERR :Integer);

implementation

procedure SF33R(X :Real; N :Integer; NMAX :Integer;
                var FUN1 :Array of Real; var FUN2 :Array of Real;
                var WORK :Array of Real; var IERR :Integer);
var
N1,N2,NA,NB,NC1,NC2,I,J,K :Integer;
P,P0,P1,Q0,Q1,Q2,R,R1,R2,R3,R4,R5,U,U1,U2,U3,U4,U5,V,V1,V2,V3,V4,V5,
   V6,V7 :Real;
const
SYS039 :Real = 0.57721566490153286060651200000;
SYS015 :Real = 0.63661977236758134307553505349006;
SYS005 :Real = 1.5707963267948966192313216916397;
label
_1,_2,_3,_4,_5,_6,_7,_8,_9,_10,_11,_12,_13,_14,_15,_16,_17,_18,_19,
_20,_21,_22,_23,_24,_25,_26,_27,_28;
begin
if ( N < 2 ) 
 then goto _26;
NC1 := Max0(N,2*Trunc(X)+4);
if ( NMAX < NC1 ) 
 then goto _28;
IERR := 0;
if ( X ) < 0
 then goto _1
 else if ( X ) > 0
       then goto _2
       else goto _1;
_1:
IERR := 67;
goto _27;
_2:
NC1 := Max1(X+1.0,3.0);
NC2 := 2*NC1+1;
if ( N-NC1 ) < 0
 then goto _3
 else if ( N-NC1 ) > 0
       then goto _4
       else goto _3;
_3:
N2 := N;
N1 := N;
goto _7;
_4:
N1 := NC1;
if ( N-NC2 ) < 0
 then goto _5
 else if ( N-NC2 ) > 0
       then goto _6
       else goto _5;
_5:
N2 := NC2+1;
goto _7;
_6:
N2 := N;
_7:
if ( X >= 10.0 ) 
 then goto _14;
if ( X > 1.0 ) 
 then goto _12;
R1 := 0.5*X;
R2 := -IntPower(R1,2);
U1 := 1.0;
U2 := 1.0;
U3 := 1.0;
U4 := 1.0;
P0 := 1.0;
P1 := 1.0;
Q0 := -(Ln(R1)+SYS039);
V1 := Q0;
Q1 := V1+0.5;
V2 := Q1;
for I:=1 to 8 do
 begin
  U1 := U1*R2/((N-1+I)*I);
  P0 := P0+U1;
  U2 := U2*R2/((N-2+I)*I);
  P1 := P1+U2;
  R3 := 1.E0/(I);
  R4 := 1.E0/(I+1);
  U3 := U3*R2*IntPower(R3,2);
  V1 := V1+R3;
  Q0 := Q0+U3*V1;
  U4 := U4*R2*R3*R4;
  V2 := V2+0.5*(R3+R4);
  Q1 := Q1+U4*V2;
_8:
 end;
FUN1[N-1] := P0;
FUN1[N-2] := P1;
R3 := -SYS015;
FUN2[0] := R3*Q0;
FUN2[1] := R3*(1.E0-2.E0*R2*Q1);
if ( N = 2 ) 
 then goto _11;
FUN2[2] := FUN2[1]+R2*FUN2[0]*2;
for I:=4 to N do
 begin
  FUN2[I-1] := FUN2[I-2]+FUN2[I-3]*R2/((I-2)*(I-3));
_9:
 end;
NA := N-2;
for J:=1 to NA do
 begin
  I := NA-J+1;
  FUN1[I-1] := FUN1[I+0]+FUN1[I+1]*R2/(I*(I+1));
_10:
 end;
_11:
goto _25;
_12:
R1 := 0.5*X;
R2 := -IntPower(R1,2);
U1 := 1.0;
U2 := 1.0;
P0 := 1.0;
P1 := 1.0;
Q0 := -(Ln(R1)+SYS039);
V1 := Q0;
Q1 := V1+0.5;
V2 := Q1;
for I:=1 to 20 do
 begin
  R3 := 1.E0/(I);
  R4 := 1.E0/(I+1);
  U1 := U1*R2*IntPower(R3,2);
  P0 := P0+U1;
  V1 := V1+R3;
  Q0 := Q0+U1*V1;
  U2 := U2*R2*R3*R4;
  P1 := P1+U2;
  V2 := V2+0.5*(R3+R4);
  Q1 := Q1+U2*V2;
_13:
 end;
FUN1[0] := P0;
FUN1[1] := R1*P1;
R2 := -SYS015;
FUN2[0] := R2*Q0;
FUN2[1] := (0.5/R1+R1*Q1)*R2;
goto _16;
_14:
R1 := 0.125/X;
R2 := -IntPower(R1,2);
P0 := 1.0;
Q0 := 1.0;
P1 := 1.0;
Q1 := 1.0;
U1 := 1.0;
U2 := 1.0;
U3 := 1.0;
U4 := 1.0;
for I:=1 to 9 do
 begin
  V1 := (2*I);
  V2 := 2.E0*V1;
  V3 := IntPower((V2-3.0)*(V2-1.0),2);
  V4 := IntPower(V2,2)-1.0;
  V5 := V4*(V2-3.0);
  V6 := V5*(V2+3.0);
  V5 := V5*(V2-5.0);
  V4 := IntPower(V4,2);
  V7 := R2/((V1-1.0)*V1);
  V2 := R2/((V1+1.0)*V1);
  U1 := U1*V3*V7;
  P0 := P0+U1;
  U2 := U2*V4*V2;
  Q0 := Q0+U2;
  U3 := U3*V5*V7;
  P1 := P1+U3;
  U4 := U4*V6*V2;
  Q1 := Q1+U4;
_15:
 end;
Q0 := R1*Q0;
Q1 := 3.E0*R1*Q1;
U1 := SYS005;
U2 := Sqrt(U1*X);
U3 := X+0.5*U1;
U4 := Sin(U3)/U2;
U5 := Cos(U3)/U2;
FUN1[0] := P0*U4-Q0*U5;
FUN1[1] := Q1*U4-P1*U5;
FUN2[0] := -(P0*U5+Q0*U4);
FUN2[1] := -(P1*U4+Q1*U5);
_16:
R1 := 2.E0/X;
for I:=3 to N1 do
 begin
  R2 := R1*(I-2);
  FUN1[I-1] := R2*FUN1[I-2]-FUN1[I-3];
  FUN2[I-1] := R2*FUN2[I-2]-FUN2[I-3];
_17:
 end;
if ( N1 = N ) 
 then goto _25;
R1 := 0.5*X;
R2 := IntPower(R1,2);
Q2 := 1.0;
U := 1.0;
V := 1.0;
R3 := R1;
NA := N2-2;
for I:=1 to NA do
 begin
  V := V+1.E0/(I+1);
  U := U*R2/(I*(NA+1-I));
  Q2 := Q2+U;
  R3 := R3*R1/(I);
_18:
 end;
U := 1.0;
P := 1.0;
R2 := -R2;
if ( X-10.0 ) < 0
 then goto _19
 else if ( X-10.0 ) > 0
       then goto _20
       else goto _20;
_19:
K := 20;
goto _21;
_20:
K := NC2-1;
_21:
Q1 := 0.5*V-Ln(R1)-SYS039;
for I:=1 to K do
 begin
  R4 := 1.E0/(I);
  R5 := 1.E0/(N2-1+I);
  U := U*R2*R4*R5;
  P := P+U;
  V := V+0.5*(R4+R5);
  Q1 := Q1+U*V;
_22:
 end;
Q2 := 0.5*Q2/R3;
R3 := R3/(N2-1);
P := P*R3;
Q1 := Q1*R3;
R2 := -SYS015;
FUN1[N2-1] := P;
FUN2[N2-1] := R2*(Q1+Q2);
R := 2.0/X;
WORK[N1-1] := 0.E0;
NA := N1+1;
NB := N2-1;
for I:=NA to NB do
 begin
  WORK[I-1] := 1.E0/(R*(I-1)-WORK[I-2]);
  FUN1[I-1] := WORK[I-1]*FUN1[I-2];
  FUN2[I-1] := WORK[I-1]*FUN2[I-2];
_23:
 end;
for J:=NA to NB do
 begin
  I := NA+NB-J;
  FUN1[I-1] := WORK[I-1]*FUN1[I+0]+FUN1[I-1];
  FUN2[I-1] := WORK[I-1]*FUN2[I+0]+FUN2[I-1];
_24:
 end;
_25:
exit;
_26:
IERR := 65;
_27:
UTSF10(IERR,33);
goto _25;
_28:
IERR := 66;
goto _27;
end;

end.
